/*	mms.h - definitions for the Linux/16 Memory Manager System (tm)
 *	(C) 1995 Chad Page
 *
 *	These definitions are *not* arch-specific, although each arch could
 *	have different mms implementations in the same build (for instance,
 *	the 80x86/16bit arch tree could have RealMode, EMS, and DPMI MMS 
 *	handlers.  Each MMS implemtation will have arch-specific code and
 * 	are intended to be only semi-portable at best.
 *	
 *	The plus is that other parts of Linux/16 that use MMS will be much
 * 	more portable...
 */

/* The __hglobal 'structure' is actually a void pointer so any handler can
 * use it's own structure */

typedef void * __hglobal;
typedef __hglobal * __phglobal;

/* MMS return codes... */
typedef __uint __mms_ret;

#define MRET_UNDEFINED 0
#define MRET_OK	1
#define MRET_NOMEM 2
#define MRET_FRAGMENTED 3

/* MMS information structure... */

struct __mms_info {
	__u16 granularity;
	__uint memunits;	/* Memory from mms = memunits * granularity */
	__uint maxcode, maxdata, maxstack; /* In bytes */
	__u32 mms_archflags;
};

/* Mode flags, in order of likelyhood of implementation :) */
/* Hopefully these are the only true Intelisms in this spec. */

#define MMSF_I86REALMODE 0x1	/* 8086 real mode */
#define MMSF_IPROTMODE16 0x2	/* 286 protected mode */
#define MMSF_FLATMEM 0x4	/* Eg 386+ 32-bit protected mode, DEC Alpha... */
#define MMSF_PAGEMODE 0x8	/* EMS will probably qualify */
#define MMSF_EMS 0x10		/* EMS will take out most mm limitations
				 * (except small model code)... */
#define MMSF_DPMI 0x20		/* DPMI will have to use the BIOS-driver
				 * package, although MMS could care less
				 * if the foobar driver package is used :) */

#define STDMEMCPYPARAM (__hglobal h, __pvoid dest, __pvoid src, __uint len)

struct _mms_ops 
{
	__pmms_info _VFUNCTION(init, ());
	__pvoid _VFUNCTION(memcpy_tohg, STDMEMCPYPARAM);
	__pvoid _VFUNCTION(memcpy_fromhg, STDMEMCPYPARAM);
	__pvoid _VFUNCTION(memcpy_hg, STDMEMCPYPARAM);
	__pvoid _VFUNCTION(memcpy_hgs, (__hglobal ha, __pvoid dest, __hglobal hb, __pvoid src, __uint len));
	__mms_ret _VFUNCTION(global_alloc, (__uint len, __u16 flags, __phglobal h)); 
	__mms_ret _VFUNCTION(global_free, (__hglobal_block));
	__mms_ret _VFUNCTION(global_lock, (__hglobal_block, __pvoid *p));
	__mms_ret _VFUNCTION(global_unlock, (__hglobal block)); 
	__mms_ret _VFUNCTION(global_getsize, (__hglobal block, __puint size));
	__mms_ret _VFUNCTION(global_resize, (__hglobal block, __uint newsize, __puint actsize));
	__mms_ret _VFUNCTION(global_compact, (__uint reqsize, __puint actsize));
	__mms_ret _VFUNCTION(prepare_process, (__ptask task));
	__mms_ret _VFUNCTION(unprepare_process, (__ptask task));
};

typedef _mms_ops __MMS_ops;
typedef _mms_ops __pMMS_ops;

/**********************************************************************
    MMS stubs (as defined in mm/mms_stubs.c)
***********************************************************************/

__pmms_info _FUNCPROTO(mms_init, ());
/* memcpy group */
__pvoid _FUNCPROTO(memcpy_tohg, STDMEMCPYPARAM);
__pvoid _FUNCPROTO(memcpy_fromhg, STDMEMCPYPARAM);   
__pvoid _FUNCPROTO(memcpy_gb, STDMEMCPYPARAM);
__pvoid _FUNCPROTO(memcpy_gbs, (__hglobal ha, __pvoid dest, __hglobal hb, __pvoid src, __uint len));
/* global block group */
__mms_ret _FUNCPROTO(mms_global_alloc, (__uint len, __u16 flags, __phglobal h));
__mms_ret _FUNCPROTO(mms_global_free, (__hglobal block)); 
__mms_ret _FUNCPROTO(mms_global_lock, (__hglobal block, __pvoid *p));
__mms_ret _FUNCPROTO(mms_global_unlock, (__hglobal block));
__mms_ret _FUNCPROTO(mms_global_getsize, (__hglobal block, __uint size));
__mms_ret _FUNCPROTO(mms_global_resize, (__hglobal block, __uint newsize, __puint actsize));
__mms_ret _FUNCPROTO(mms_global_compact, (__uint reqsize, __puint actsize));  
/* Process group */
__mms_ret _FUNCPROTO(mms_prepare_process, (__ptask task));
__mms_ret _FUNCPROTO(mms_unprepare_process, (__ptask task));

